@@ -3,13 +3,14 @@ use strict;
 use warnings;
 
 use Config;
+use Carp qw/croak/;
 
 sub import {
     my $class = shift;
     my $caller = caller;
 
     for my $check (@_) {
-        die "'$check' is not a known capability"
+        croak "'$check' is not a known capability"
             unless $check =~ m/^CAN_/ && $class->can("$check");
 
         my $const = get_const($check);
@@ -45,6 +46,12 @@ sub CAN_THREAD {
     return 0 unless $] >= 5.008001;
     return 0 unless $Config{'useithreads'};
 
+    # Threads are broken on perl 5.10.0 built with gcc 4.8+
+    if ($] == 5.010000 && $Config{'ccname'} eq 'gcc' && $Config{'gccversion'}) {
+        my @parts = split /\./, $Config{'gccversion'};
+        return 0 if $parts[0] >= 4 && $parts[1] >= 8; 
+    }
+
     # Change to a version check if this ever changes
     return 0 if $INC{'Devel/Cover.pm'};
     return 1;
