@@ -5,7 +5,7 @@ use warnings;
 use Scalar::Util qw/blessed/;
 use Carp qw/confess/;
 
-use Test::Stream::TAP qw/OUT_STD OUT_TODO OUT_ERR/;
+use Test::Stream::Formatter::TAP qw/OUT_STD OUT_TODO OUT_ERR/;
 
 use Test::Stream::Event::Diag;
 
@@ -176,6 +176,11 @@ this arrayref.
 This is the true/false value of the test after TODO, SKIP, and similar
 modifiers are taken into account.
 
+=item $b = $e->allow_bad_name
+
+This relaxes the test name checks such that they allow characters that can
+confuse a TAP parser.
+
 =back
 
 =head1 METHODS
@@ -196,7 +201,7 @@ This generates the default diagnostics string:
 Generate the tap stream for this object. C<@sets> containes 1 or more arrayrefs
 that identify the IO handle to use, and the string that should be sent to it.
 
-IO Handle identifiers are set to the value of the L<Test::Stream::TAP> C<OUT_*>
+IO Handle identifiers are set to the value of the L<Test::Stream::Formatter::TAP> C<OUT_*>
 constants.
 
 Example:
