@@ -3,11 +3,26 @@ use strict;
 use warnings;
 
 use Test::Stream::Capabilities qw/CAN_THREAD/;
-use Scalar::Util qw/reftype blessed/;
+use Scalar::Util qw/reftype blessed refaddr/;
 use Carp qw/croak/;
 
 use Test::Stream::Exporter qw/import export_to exports/;
-exports qw{ try protect get_tid USE_THREADS };
+exports qw{
+        try protect
+
+        get_tid USE_THREADS
+
+        pkg_to_file
+
+        get_stash
+
+        sig_to_slot slot_to_sig
+        parse_symbol
+
+        term_size
+
+        rtype render_ref
+};
 no Test::Stream::Exporter;
 
 sub _manual_protect(&) {
@@ -53,7 +68,12 @@ sub _manual_try(&;@) {
         }
 
         ($@, $!) = ($msg, $no);
-        $SIG{__DIE__} = $die;
+        if ($die) {
+            $SIG{__DIE__} = $die;
+        }
+        else {
+            delete $SIG{__DIE__};
+        }
     }
 
     return ($ok, $error);
@@ -111,6 +131,87 @@ BEGIN {
     }
 }
 
+sub pkg_to_file {
+    my $pkg = shift;
+    my $file = $pkg;
+    $file =~ s{(::|')}{/}g;
+    $file .= '.pm';
+    return $file;
+}
+
+sub get_stash {
+    my $pkg = shift;
+    no strict 'refs';
+    return \%{"$pkg\::"};
+}
+
+my %SIG_TABLE = (
+    '&' => 'CODE',
+    '%' => 'HASH',
+    '@' => 'ARRAY',
+    '$' => 'SCALAR',
+    '*' => 'GLOB',
+);
+my %SLOT_TABLE = reverse %SIG_TABLE;
+
+sub sig_to_slot { $SIG_TABLE{$_[0]}  }
+sub slot_to_sig { $SLOT_TABLE{$_[0]} }
+
+sub parse_symbol {
+    my ($sym) = @_;
+
+    my $sig = substr($sym, 0, 1);
+    return ($sym, 'CODE') unless $SIG_TABLE{$sig};
+    my $name = substr($sym, 1);
+
+    my $slot = $SIG_TABLE{$sig} or croak "'$sig' is not a supported sigil";
+
+    return ($name, $slot);
+}
+
+BEGIN {
+    my ($ok, $err) = try { require Term::ReadKey };
+    $ok &&= Term::ReadKey->can('GetTerminalSize');
+    *USE_TERM_READKEY = $ok ? sub() { 1 } : sub() { 0 };
+};
+
+sub term_size {
+    return $ENV{TS_TERM_SIZE} if $ENV{TS_TERM_SIZE};
+    return 80 unless USE_TERM_READKEY;
+    my ($total) = Term::ReadKey::GetTerminalSize(*STDOUT);
+    return 80 if !$total;
+    return 80 if $total < 80;
+    return $total;
+}
+
+sub rtype {
+    my ($thing) = @_;
+    return '' unless defined $thing;
+
+    my $rf = ref $thing;
+    my $rt = reftype $thing;
+
+    return '' unless $rf || $rt;
+    return 'REGEXP' if $rf =~ m/Regex/i;
+    return 'REGEXP' if $rt =~ m/Regex/i;
+    return $rt || '';
+}
+
+sub render_ref {
+    my ($in) = @_;
+
+    my $type = rtype($in);
+    return "$in" unless $type;
+
+    # Look past overloading
+    my $class = blessed($in) || '';
+    my $it = sprintf('0x%x', refaddr($in));
+    my $ref = "$type($it)";
+
+    return $ref unless $class;
+    return "$class=$ref";
+}
+
 1;
 
 __END__
@@ -170,6 +271,39 @@ Returns true if threads are enabled, false if they are not.
 This will return the id of the current thread when threads are enabled,
 otherwise it returns 0.
 
+=item my $file = pkg_to_file($package)
+
+Convert a package name to a filename.
+
+=item $stash = get_stash($package)
+
+Returns the stash reference for the given package. The stash reference can be
+treated like a hashref, you can get keys and values from it.
+
+=item $slot = sig_to_slot($sigil)
+
+Given a sigil such as C<$>, C<@>, C<%>, C<&>, C<*>, this will return the GLOB
+slot for that sigil such as C<SCALAR>, C<ARRAY>, C<HASH>, C<CODE>, C<GLOB>.
+
+=item $sigil = slot_to_sig($slot)
+
+Given a a glob slot such as C<SCALAR>, C<ARRAY>, C<HASH>, C<CODE>, C<GLOB>,
+this will return the typical sigil for that slot such as C<$>, C<@>, C<%>,
+C<&>, C<*>.
+
+=item ($name, $type) = parse_symbol($symbol)
+
+When given a symbol name such as C<$foo> or C<@bar> this will return the symbol
+name, and the type name. If no sigil is present in the variable name it will
+assume it is a subroutine and return the C<CODE> type. C<$symbol> should be a
+string containing the name of the symbol with optional sigil.
+
+=item my $cols = term_size()
+
+Attempts to find the width in columns (characters) of the current terminal.
+Returns 80 as a safe bet if it cannot find it another way. This is most
+accurate if L<Term::ReadKey> is installed.
+
 =back
 
 =head1 SOURCE
